The key frame is meant as a part of stream such that
the stream can be decoded from the beginning of each key
frame without knowledge of the stream content before that
point.
The key frame is meant as a part of stream such that
the stream can be decoded from the beginning of each key
frame without knowledge of the stream content before that
point.