mix membrane.gen.endpoint (Membrane Core v1.3.0)

Copy Markdown View Source

Generates a template for a Membrane Endpoint with the provided module name.

$ mix membrane.gen.endpoint module_name [-l target_location]

Options

  • -l, --location - If a target location is provided, the Endpoint will be created there, relative to the lib directory. The filename must also be present and have an .ex extension. If location is not provided, then it will be inferred from the provided module name - it will be converted to lowercase and . separators will be interpreted as directory separators. For example, a Endpoint with module name Foo.Bar will be created at lib/foo/bar.ex.

Summary

Functions

do_run(base_dir, argv)

@spec do_run(binary(), [binary()]) :: any()