Generates a template for a Membrane Source with the provided module name.
$ mix membrane.gen.source module_name [-l target_location]
Options
-l, --location- If a target location is provided, the Source will be created there, relative to thelibdirectory. The filename must also be present and have an.exextension. If location is not provided, then it will be inferred from the provided module name - it will be converted to lowercase and.separators will be interpreted as directory separators. For example, a Source with module nameFoo.Barwill be created atlib/foo/bar.ex.